unit MTFilter1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, kbmMemTable, StdCtrls;

type
  TForm1 = class(TForm)
    OnFilterBtn: TButton;
    MT: TkbmMemTable;
    MTRef: TIntegerField;
    RangeBtn: TButton;
    IndexBtn: TButton;
    FilterIndexBtn: TButton;
    FilterBtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure OnFilterBtnClick(Sender: TObject);
    procedure RangeBtnClick(Sender: TObject);
    procedure IndexBtnClick(Sender: TObject);
    procedure FilterIndexBtnClick(Sender: TObject);
    procedure FilterBtnClick(Sender: TObject);
  private
    procedure FilterRecord(DataSet: TDataSet; var Accept: Boolean);
    procedure FilterIndex(DataSet: TDataSet; Index: TkbmIndex; var Accept: Boolean);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
   n: integer;
   t: cardinal;
begin
   AllocConsole;

   t := GetTickCount;
   MT.Open;
   MT.EnableIndexes := False;
   for n := 1 to 200000 do begin
      MT.Append;
      MTRef.Value := Random(200000);
   end;
   MT.Post;
   WriteLn('Table filled in         ', GetTickCount - t, ' ms');

   t := GetTickCount;
   MT.EnableIndexes := True;
   MT.UpdateIndexes;
   WriteLn('Index created in        ', GetTickCount - t, ' ms');

   t := GetTickCount;
   MT.AddFilteredIndex('FilterIndex', 'Ref', [], '', [], FilterIndex);
   WriteLn('Filter index created in ', GetTickCount - t, ' ms');

   {  Note that you also can make a filtered index by specifying the filter
      expression without using the FilterIndex procedure, like this:
      MT.AddFilteredIndex('FilterIndex', 'Ref', [], '(Ref >= 10000) and (Ref <= 50000)', [], nil);
      But this will drastisch slow down the build of the index, because the
      filter expression has to be evealuated over and over again. }
end;

procedure TForm1.FilterIndex(DataSet: TDataSet; Index: TkbmIndex; var Accept: Boolean);
begin
   Accept := (MTRef.Value >= 10000) and (MTRef.Value <= 50000);
end;

procedure TForm1.FilterRecord(DataSet: TDataSet; var Accept: Boolean);
begin
   Accept := (MTRef.Value >= 10000) and (MTRef.Value <= 50000);
end;

procedure TForm1.FilterBtnClick(Sender: TObject);
var
   n: integer;
   t: cardinal;
begin
   t := GetTickCount;
   MT.Filter   := '(Ref >= 10000) and (Ref <= 50000)';
   MT.Filtered := True;
   n := 0;
   MT.First;
   while not MT.Eof do begin
      Inc(n);
      MT.Next;
   end;
   WriteLn('Filter:       ', n, ' Records done in ', GetTickCount - t, ' ms');
   MT.Filtered := False;
   MT.Filter   := '';
end;

procedure TForm1.OnFilterBtnClick(Sender: TObject);
var
   n: integer;
   t: cardinal;
begin
   t := GetTickCount;
   MT.OnFilterRecord := FilterRecord;
   MT.Filtered := True;
   n := 0;
   MT.First;
   while not MT.Eof do begin
      Inc(n);
      MT.Next;
   end;
   WriteLn('OnFilter:     ', n, ' Records done in ', GetTickCount - t, ' ms');
   MT.Filtered := False;
   MT.OnFilterRecord := nil;
end;

procedure TForm1.RangeBtnClick(Sender: TObject);
var
   n: integer;
   t: cardinal;
begin
   t := GetTickCount;
   MT.SetRange([10000], [50000]);
   n := 0;
   MT.First;
   while not MT.Eof do begin
      Inc(n);
      MT.Next;
   end;
   WriteLn('Range:        ', n, ' Records done in ', GetTickCount - t, ' ms');
   MT.CancelRange;
end;

procedure TForm1.IndexBtnClick(Sender: TObject);
var
   n: integer;
   t: cardinal;
begin
   t := GetTickCount;
   n := 0;
   MT.FindNearest([10000]);
   while MTRef.Value <= 50000 do begin
      Inc(n);
      MT.Next;
   end;
   WriteLn('Index:        ', n, ' Records done in ', GetTickCount - t, ' ms');
end;

procedure TForm1.FilterIndexBtnClick(Sender: TObject);
var
   n: integer;
   t: cardinal;
begin
   t := GetTickCount;
   MT.IndexName := 'FilterIndex';
   n := 0;
   MT.First;
   while not MT.Eof do begin
      Inc(n);
      MT.Next;
   end;
   WriteLn('Filter index: ', n, ' Records done in ', GetTickCount - t, ' ms');
   MT.IndexName := 'MTIndex1';
end;

end.
